/*
	Copyright (c) 2001 Maurice Gittens <maurce@gittens.nl>

   This software is distributed under the GNU General
   Public License.
	See the file COPYING for more information on usage and
	redistribution of this file and for a DISCLAIMER OF ALL
	WARRANTIES.

	Written by Maurice Gittens <maurice@gittens.nl>.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <tiny_parserGenerator.h>

#include <iostream>
#include <stdexcept>
#include <stdlib.h>
#include <cstdio>
#include <unistd.h>

using namespace std;

//
// 	Static functions
//
static void handle_arguments(int argc, char *argv[]);
static void banner();
static void usage();

//
//
//
static const char* inputFile;
static char* sourceFile;
static char* headerFile;
static bool printTables;
static bool printActions;
static bool printBanner;

//
//	Main
//
int main(int argc, char *argv[])
{
	try
	{
	   handle_arguments(argc, argv);

      if (printBanner)
		   banner();

      tiny_parserGenerator pg(inputFile, sourceFile, headerFile);

      if (printTables)
         pg.setPrintTables();
      if (printActions)
         pg.setPrintActions();

	   if (pg.run())
	   {
	      unlink(sourceFile);
	      unlink(headerFile);
         return 1;
	   }
	}
	catch(exception& e)
	{
      unlink(sourceFile);
      unlink(headerFile);
   	return 2;
	}

	return 0;
}

//
//
//
static void banner()
{


   static const char *text =
	"\nThe Tiny parser generator version " VERSION "\n"
	"Copyright (c) 2001 Maurice Gittens <maurice@gittens.nl> All Rights Reserved. \n"
	"This software is covered by the GNU General Public Licence. \n";

	cout << text << endl;
	exit(1);
}

//
//
//
static void usage()
{
	static const char *text =
	"tiny [options] inputfile [outputsourcefile outputheaderfile]\n"
   "-t	print parsetables\n"
   "-v	print banner and exit\n"
   "-a	print semantic actions\n"
   "\n"
	"\n";

	cout << text << endl;
   exit(1);
}

static void handle_arguments(int argc, char *argv[])
{

   while( --argc > 0 && **++argv == '-' )
   {
      char* s;
      for( s = argv[0]+1; *s ; s++ )
      {
         switch ( *s )
         {
            case 'a' :
               printActions = true;
            break;
            case 'v' :
               printBanner = true;
            break;
            case 't' :
               printTables = true;
            break;
            case '?' :
           default:
               usage();
         } /* endswitch */
         break;
      }
   }

   if (argc == 1)
   {
      inputFile = *argv;
      int len = strlen(inputFile) + 32;

      sourceFile = new char[len];
      headerFile = new char[len];

      snprintf(sourceFile, len, "%s.cpp", inputFile);
      snprintf(headerFile, len, "%s.h", inputFile);
   }
   else if (argc == 3)
   {
      inputFile = *argv;
      sourceFile = argv[1];
      headerFile = argv[2];
   }
   else
      usage();
}


