/*
	Copyright (c) 2001 Maurice Gittens <maurce@gittens.nl>

   This software is distributed under the GNU General
   Public License.
	See the file COPYING for more information on usage and
	redistribution of this file and for a DISCLAIMER OF ALL
	WARRANTIES.

	Written by Maurice Gittens <maurice@gittens.nl>.
*/

#ifndef TINY_CODEGENERATOR_H
#define TINY_CODEGENERATOR_H

#include <fstream>

class tiny_rewritableSymbol;
class tiny_grammarSymbol;
class tiny_action;
class tiny_lexer;
class tiny_parserGenerator;
class tiny_symbolString;

class tiny_codeGenerator
{
public:
	tiny_codeGenerator(tiny_parserGenerator& p,
	                   tiny_lexer& sc,
	                   std::ofstream &sf,
	                   std::ofstream &hf)
	: pg(p),
	  scanner(sc),
     source(sf),
     header(hf)
	{
	   reset();
	}

	void operator()() { generateCode(); }

	void genIncludeHeader();
	void genIncludeFooter();
	void genCodeHeader();
	void genCodeFooter();
	void generateCode();
	void genClassName();
	void genHeaderTokens(const char *s);

	void reset() { offset = 0; }

	void genLineInfo(int,int);

protected:
	void genRHSTable();
	void traverse(void (tiny_codeGenerator::*)(const tiny_action&));
	void traverse(void (tiny_codeGenerator::*)(const tiny_grammarSymbol&));
	void traverse(void (tiny_codeGenerator::*)(tiny_rewritableSymbol &));
	void traverse(void (tiny_codeGenerator::*)(const tiny_rewritableSymbol &));

	void genConstants();

	void genNamesOfSymbols();
	void genGrammarSymbolName(const tiny_grammarSymbol&);
	void genTerminalName(const tiny_grammarSymbol&);
	void genNonTerminalName(const tiny_rewritableSymbol &);

   void genGenerics();
	void genGenerics(const tiny_rewritableSymbol&);
   void genGenericActionOffsetTable();
	void genGenericActionOffset(const tiny_rewritableSymbol&);
	void genNonTerminalOffsetTable();
	void genNonTerminalOffset(const tiny_rewritableSymbol&);
	void genCSRuleTable();
	void genCSRule(tiny_rewritableSymbol&);
	void genCSRuleOffsetTable();
	void genCSRuleOffset(tiny_rewritableSymbol&);

	void genActions();
	void genAction(const tiny_action &);

	void genTemplatePrefix();
	void genCharTemplatePrefix();

private:
   tiny_codeGenerator(const tiny_codeGenerator&);
   tiny_codeGenerator& operator=(const tiny_codeGenerator&);

   void emitSymbolString(const tiny_symbolString &);
	
   int offset;
   tiny_parserGenerator& pg;
   tiny_lexer& scanner;
   std::ofstream& source;
   std::ofstream& header;
};


#endif
