/*
	Copyright (c) 2001 Maurice Gittens <maurce@gittens.nl>

   This software is distributed under the GNU General
   Public License.
	See the file COPYING for more information on usage and
	redistribution of this file and for a DISCLAIMER OF ALL
	WARRANTIES.

	Written by Maurice Gittens <maurice@gittens.nl>.
*/

#ifndef TINY_PARSERGENERATOR_SEEN
#define TINY_PARSERGENERATOR_SEEN

#include <tiny_parser.h>
#include <tiny_lexer.h>
#include <tiny_symbolprocessor.h>
#include <tiny_codegenerator.h>

#include <fstream>

//
//
//
class tiny_parserGenerator
{
public:
	tiny_parserGenerator
	(
	   const char* ifn,
	   const char* sfn,
	   const char* hfn,
	   int bs = 1024,
		int ts = 1024)
   : inputFileName(ifn),
     sourceFileName(sfn),
     headerFileName(hfn),
	  sourceOutputStream(sfn),
     headerOutputStream(hfn),
     scanner(ifn, sourceOutputStream, headerOutputStream, bs),
     parser(scanner),
     symbolProcessor(scanner),	
     codeGenerator(*this, scanner, sourceOutputStream, headerOutputStream)
   {
      init();
   }
	
	~tiny_parserGenerator() {}

	void reset();

	void verifyParseData();
	void setPrintActions() { printActions = true; }
	bool getPrintActions() const { return printActions; }
	void setPrintTables() { printTable = true; }
	bool getPrintTables() const { return printTable; }
	void setCheckTerminals() { checkTerminals = 1; }
	bool  getCheckTerminals() { return checkTerminals; }
	void setDontSortRules() { sortRules = 0; }
	int getSortRules() const { return sortRules; }
	void setLineNumbers() { lineNumbers = 1; }
	int  getLineNumbers() { return lineNumbers; }

	tiny_lexer &getScanner() { return scanner; }
	tiny_parser &getParser() { return parser; }

	tiny_symbolProcessor& getSymbolProcessor() { return symbolProcessor; }
	tiny_codeGenerator& getCodeGenerator() { return codeGenerator; }

	std::ofstream& getOutputStream() { return sourceOutputStream; }
	std::ofstream& getHeaderStream() { return headerOutputStream; }

	void setInputFile(const std::string&);
	void setOutputFile(const std::string&);

	const std::string& getHeaderFileName() const
	{
		return headerFileName;
	}

	const std::string& getInputFileName() const
	{
		return inputFileName;
	}

	int run();

protected:
	void initHeaderFileName(const std::string&);
	void initOutputFileName(const std::string&);

private:
   tiny_parserGenerator(const tiny_parserGenerator&);
   tiny_parserGenerator operator=(const tiny_parserGenerator&);

	std::string inputFileName;
	std::string sourceFileName;
	std::string headerFileName;
	std::ofstream sourceOutputStream;
	std::ofstream headerOutputStream;

	tiny_lexer scanner;
	tiny_parser parser;
	tiny_symbolProcessor symbolProcessor;
	tiny_codeGenerator codeGenerator;

 	bool printTable;
	bool printActions;
	bool checkTerminals;
	bool lineNumbers;
	bool sortRules;

	void init()
	{
		// reset local variables
		lineNumbers = checkTerminals = printActions =
		printTable = 0; sortRules = 1;
	}

};

#endif


