/*
	Copyright (c) 2001 Maurice Gittens <maurce@gittens.nl>

   This software is distributed under the GNU General
   Public License.
	See the file COPYING for more information on usage and
	redistribution of this file and for a DISCLAIMER OF ALL
	WARRANTIES.

	Written by Maurice Gittens <maurice@gittens.nl>.
*/

#ifndef TINY_REWRITABLESYMBOL_SEEN
#define TINY_REWRITABLESYMBOL_SEEN

#include <tiny_grammarsymbol.h>
#include <tiny_principle.h>
#include <tiny_symbolstring.h>

/**
 *
 */
class tiny_rewritableSymbol : public tiny_grammarSymbol
{
public:
	tiny_rewritableSymbol(Type t,
	                      const std::string& n,
	                      unsigned int i,
	                      unsigned ln)
	: tiny_grammarSymbol(t,n,i,ln), offset(0)
	{
	}

	tiny_rewritableSymbol& addRule(const tiny_principle& r)
	{
		rules.push_back(r);
		rules.back().setId(rules.size());
		return *this;
	}

	virtual unsigned int getEncodedId() const { return makeNonTerminal(id); }
	
	unsigned int getRuleCount() const { return rules.size(); }
	const tiny_principle &getRule(int i) const { return rules[i]; }
	tiny_principle &getRule(int i) { return rules[i]; }
	const std::vector<tiny_principle>& getRules() const { return rules; }
	std::vector<tiny_principle>& getRules() { return rules; }
	const tiny_symbolString& getLHS(int x) const { return rules[x].getLHS(); }
	tiny_symbolString& getLHS(int x) { return rules[x].getLHS(); }
	tiny_symbolString& getRHS(int x, int y) { return rules[x].getRHS(y).getString(); }

	unsigned int getOffset() const { return offset; }
	void setOffset(int o) { offset = o; }

   void setFirstSetCapacity(unsigned int s)
   {
      for(std::vector<tiny_principle>::iterator it = rules.begin();
          it != rules.end();
          ++it)
      {
         it->setFirstSetCapacity(s);
      }
   }

	int SameLhs();  // this function is intended to return
			// if to left hand sides are found to be
			// identical

private:
	std::vector<tiny_principle> rules;
	unsigned offset;
};

#endif
