/*
	Copyright (c) 2001 Maurice Gittens <maurce@gittens.nl>

   This software is distributed under the GNU General
   Public License.
	See the file COPYING for more information on usage and
	redistribution of this file and for a DISCLAIMER OF ALL
	WARRANTIES.

	Written by Maurice Gittens <maurice@gittens.nl>.
*/

#include <iostream.h>
#include <string.h>
#include <tiny_symbolstring.h>

//
//
//
bool tiny_symbolString::isNullable() const
{
	int len = length();
	int dotIndex = getDotIndex();
	return len == (dotIndex + 1) ||
		   (len == (dotIndex + 2) && isAction(getSymbol(dotIndex + 1)));
}

//
//
//
bool tiny_symbolString::match(tiny_symbolString& configurationString) const
{
	if (configurationString.getSymbolCount() < getSymbolCount())
		return false;

	if (configurationString.truePreContextLength() < truePreContextLength())
		return false;

	if (configurationString.truePostContextLength() < truePostContextLength())
		return false;

	int i,j,kIndex;
	int keyIndex = getDotIndex();
	int configurationIndex = configurationString.getDotIndex();

	i = configurationIndex - 1;
	j = keyIndex - 1;
	for(kIndex = 0; kIndex < keyIndex; kIndex++)
	{
		if (configurationString.getSymbol(i) != getSymbol(j))
			return false;
		i--;
		j--;
	}

	int keyLength = length();
	i = configurationIndex + 1;
	j = keyIndex + 1;
	while(i < keyLength)
	{
		if (configurationString.getSymbol(i) != getSymbol(j))
		{
		   if (isAction(getSymbol(i)) || isAction(getSymbol(j)))
			break;
		   else
			return false;
		}

		i++;
		j++;
	}

	return true;
}

//
//
//
unsigned int tiny_symbolString::truePostContextLength() const
{
	int length = sp;
	unsigned int rv = length - getDotIndex() - 1;

	if (isAction(getSymbol(length - 1)))
		rv--;

	return rv;
}

//
//
//
int getDotIndex(const tiny_symbolString * s)
{
	const int length = s->length();

	for(int i = 0; i < length; i++)
		if (!s -> getSymbol(i))
			return i;

	return -1;
}

