/*
	Copyright (c) 2001 Maurice Gittens <maurce@gittens.nl>

   This software is distributed under the GNU General
   Public License.
	See the file COPYING for more information on usage and
	redistribution of this file and for a DISCLAIMER OF ALL
	WARRANTIES.

	Written by Maurice Gittens <maurice@gittens.nl>.
*/

#ifndef TINY_SYMBOLSTRING_SEEN
#define TINY_SYMBOLSTRING_SEEN

#include <tiny_util.h>
#include <vector>
class tiny_symbolString;
int getDotIndex(const tiny_symbolString*);

//
//
//
class tiny_symbolString
{
public:
	tiny_symbolString() : sp(0) {  }

	bool empty() const { return symbols.size() == 0; }
	void push_back(unsigned short s) { symbols.push_back(s); ++sp; }
	void setSymbol(int index, unsigned short i) { symbols[index] = i; }
	unsigned short getSymbol(int i) const { return symbols[i]; }
	unsigned int length() const { assert(symbols.size() == sp); return symbols.size(); }
	unsigned int truePreContextLength() const { int rv = getDotIndex(); return isAction(getSymbol(0)) ? rv - 1 : rv; }
	unsigned int truePostContextLength() const;
	unsigned int getSymbolCount() const
	{
		int  rv = sp;

		if (isAction(getSymbol(0)))
			rv--;

		if (sp > 1 && isAction(sp - 1))
			rv--;

		return rv;
	}

	int getDotIndex() const { return ::getDotIndex(this); }
	bool match(tiny_symbolString&) const;
	bool isNullable() const;

	void advanceDot(int i)
	{
		int aux;

		aux = symbols[i];
		symbols[i] = symbols[i + 1];
		symbols[i + 1] = aux;
	}

	tiny_symbolString& clear()
	{
		symbols.clear();
		sp = 0;
		return *this;
	}

	const unsigned short *getSymbolAddress(int i) const { return &symbols[i]; }
	int hasActionSuffix() const { return (sp > 1 && isAction(sp - 1)) ? 1 : 0; }
private:
	std::vector<unsigned short> symbols;
	unsigned int sp;
};

#endif
